@@ -34,7 +34,7 @@
 #include <sys/types.h>
 #include <sys/uio.h>
 
-#ifdef __linux__
+#if defined(__linux__) || defined(__GNU__)
 #include <netinet/tcp.h>
 #if !defined(TCP_CORK)
 #undef HAVE_SENDFILE
@@ -69,6 +69,10 @@
 #include "Xrd/XrdTrace.hh"
 
 #include "XrdSys/XrdSysError.hh"
+
+#ifndef ETIME
+#define ETIME ETIMEDOUT
+#endif
   
 /******************************************************************************/
 /*                               G l o b a l s                                */
