@@ -36,7 +36,7 @@
 #include <sys/types.h>
 #include <sys/uio.h>
 
-#ifdef __linux__
+#if defined(__linux__) || defined(__GNU__)
 #include <netinet/tcp.h>
 #if !defined(TCP_CORK)
 #undef HAVE_SENDFILE
@@ -45,7 +45,7 @@
 
 #ifdef HAVE_SENDFILE
 
-#ifndef __APPLE__
+#if defined(__solaris__) || defined(__linux__) || defined(__GNU__)
 #include <sys/sendfile.h>
 #endif
 
@@ -79,12 +79,6 @@ extern XrdScheduler   Sched;
 extern XrdTlsContext *tlsCtx;
        XrdTcpMonPin  *TcpMonPin = 0;
 extern int            devNull;
-
-#ifdef IOV_MAX
-       int            maxIOV = IOV_MAX;
-#else
-       int            maxIOV = sysconf(_SC_IOV_MAX);
-#endif
 };
 
 using namespace XrdGlobal;
@@ -554,6 +548,18 @@ int XrdLinkXeq::Send(const char *Buff, int Blen)
 int XrdLinkXeq::Send(const struct iovec *iov, int iocnt, int bytes)
 {
    int retc;
+   static int maxIOV = -1;
+   if (maxIOV == -1) {
+#ifdef _SC_IOV_MAX
+      maxIOV = sysconf(_SC_IOV_MAX);
+      if (maxIOV == -1)
+#endif
+#ifdef IOV_MAX
+         maxIOV = IOV_MAX;
+#else
+         maxIOV = 1024;
+#endif
+   }
 
 // Get a lock and assume we will be successful (statistically we are)
 //
@@ -602,12 +608,12 @@ int XrdLinkXeq::Send(const struct iovec *iov, int iocnt, int bytes)
 
 int XrdLinkXeq::Send(const sfVec *sfP, int sfN)
 {
-#if !defined(HAVE_SENDFILE) || defined(__APPLE__)
+#if !defined(HAVE_SENDFILE)
+
    return -1;
 
-#else
+#elif defined(__solaris__)
 
-#ifdef __solaris__
     sendfilevec_t vecSF[XrdOucSFVec::sfMax], *vecSFP = vecSF;
     size_t xframt, totamt, bytes = 0;
     ssize_t retc;
@@ -668,7 +674,7 @@ do{retc = sendfilev(LinkInfo.FD, vecSFP, sfN, &xframt);
    Log.Emsg("Link", retc, "send file to", ID);
    return -1;
 
-#elif defined(__linux__)
+#elif defined(__linux__) || defined(__GNU__)
 
    static const int setON = 1, setOFF = 0;
    ssize_t retc = 0, bytesleft;
@@ -723,7 +729,11 @@ do{retc = sendfilev(LinkInfo.FD, vecSFP, sfN, &xframt);
    AtomicAdd(BytesOut, xfrbytes);
    wrMutex.UnLock();
    return xfrbytes;
-#endif
+
+#else
+
+   return -1;
+
 #endif
 }
 
@@ -903,7 +913,7 @@ bool XrdLinkXeq::setTLS(bool enable, XrdTlsContext *ctx)
 // We want to initialize TLS, do so now.
 //
    if (!ctx) ctx = tlsCtx;
-   eNote = tlsIO.Init(*ctx, PollInfo.FD, rwMode, hsMode, false, ID);
+   eNote = tlsIO.Init(*ctx, PollInfo.FD, rwMode, hsMode, false, false, ID);
 
 // Check for errors
 //
