@@ -18,11 +18,12 @@
 
 #include "XrdCl/XrdClSocket.hh"
 #include "XrdCl/XrdClUtils.hh"
-#include "XrdNet/XrdNetConnect.hh"
 #include "XrdCl/XrdClConstants.hh"
 #include "XrdCl/XrdClMessage.hh"
 #include "XrdCl/XrdClDefaultEnv.hh"
 #include "XrdCl/XrdClTls.hh"
+#include "XrdNet/XrdNetConnect.hh"
+#include "XrdSys/XrdSysFD.hh"
 
 #include <arpa/inet.h>
 #include <unistd.h>
@@ -41,7 +42,7 @@
 namespace XrdCl
 {
   Socket::Socket( int socket, SocketStatus status ):
-    pSocket(socket), pStatus( status ), pServerAddr( 0 ),
+    pSocket(socket), pStatus( status ),
     pProtocolFamily( AF_INET ),
     pChannelID( 0 ),
     pCorked( false )
@@ -64,7 +65,7 @@ namespace XrdCl
     if( pSocket != -1 )
       return XRootDStatus( stError, errInvalidOp );
 
-    pSocket = ::socket( family, SOCK_STREAM, 0 );
+    pSocket = XrdSysFD_Socket( family, SOCK_STREAM, 0 );
     if( pSocket < 0 )
     {
       pSocket = -1;
@@ -214,7 +215,7 @@ namespace XrdCl
     if( pSocket == -1 || pStatus == Connected || pStatus == Connecting )
       return XRootDStatus( stError, errInvalidOp );
 
-    pServerAddr = addr;
+    pServerAddr.reset( new XrdNetAddr( addr ) );;
 
     //--------------------------------------------------------------------------
     // Make sure TLS is off when the physical connection is newly established
@@ -224,8 +225,8 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     // Connect
     //--------------------------------------------------------------------------
-    int status = XrdNetConnect::Connect( pSocket, pServerAddr.SockAddr(),
-                                         pServerAddr.SockSize(), timeout );
+    int status = XrdNetConnect::Connect( pSocket, pServerAddr->SockAddr(),
+                                         pServerAddr->SockSize(), timeout );
     if( status != 0 )
     {
       XRootDStatus st( stError );
@@ -462,7 +463,7 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     // We use send with MSG_NOSIGNAL to avoid SIGPIPEs on Linux
     //--------------------------------------------------------------------------
-#ifdef __linux__
+#if defined(__linux__) || defined(__GNU__) || (defined(__FreeBSD_kernel__) && defined(__GLIBC__))
     int status = ::send( pSocket, buffer, size, MSG_NOSIGNAL );
 #else
     int status = ::write( pSocket, buffer, size );
@@ -475,6 +476,24 @@ namespace XrdCl
     return XRootDStatus();
   }
 
+  //------------------------------------------------------------------------
+  //! Write data from a kernel buffer to the socket
+  //!
+  //! @param kbuff : data to be written
+  //! @return      : the amount of data actually written
+  //------------------------------------------------------------------------
+  XRootDStatus Socket::Send( XrdSys::KernelBuffer &kbuff )
+  {
+    if( pTls ) return XRootDStatus( stError, errNotSupported, 0,
+                                     "Cannot send a kernel-buffer over TLS." );
+
+    ssize_t status = XrdSys::Send( pSocket, kbuff );
+
+    if( status <= 0 )
+      return ClassifyErrno( errno );
+    return XRootDStatus();
+  }
+
   //----------------------------------------------------------------------------
   // Poll the descriptor
   //----------------------------------------------------------------------------
@@ -759,12 +778,13 @@ namespace XrdCl
   // Enable encryption
   //------------------------------------------------------------------------
   XRootDStatus Socket::TlsHandShake( AsyncSocketHandler *socketHandler,
-                                   const std::string  &thehost )
+                                     const std::string  &thehost )
   {
     try
     {
+      if( !pServerAddr ) return XRootDStatus( stError, errInvalidOp );
       if( !pTls ) pTls.reset( new Tls( this, socketHandler ) );
-      return pTls->Connect( thehost, &pServerAddr );
+      return pTls->Connect( thehost, pServerAddr.get() );
     }
     catch( std::exception& ex )
     {
