@@ -34,6 +34,7 @@
 #include "Xrd/XrdBuffer.hh"
 #include "Xrd/XrdLink.hh"
 #include "XrdOuc/XrdOucStream.hh"
+#include "XrdOuc/XrdOucEnv.hh"
 #include "XrdOuc/XrdOucUtils.hh"
 #include "XrdSec/XrdSecProtect.hh"
 #include "XrdSfs/XrdSfsFlags.hh"
@@ -147,7 +148,7 @@ char                  XrdXrootdProtocol::tlsNot   = 0;
   
 XrdObjectQ<XrdXrootdProtocol>
             XrdXrootdProtocol::ProtStack("ProtStack",
-                                       "xrootd protocol anchor");
+                                       "xroot protocol anchor");
 
 /******************************************************************************/
 /*                       P r o t o c o l   L o a d e r                        */
@@ -170,22 +171,21 @@ XrdProtocol *XrdgetProtocol(const char *pname, char *parms,
 
 // Put up the banner
 //
-   pi->eDest->Say("Copr.  2012 Stanford University, xrootd protocol "
+   pi->eDest->Say("Copr.  2012 Stanford University, xroot protocol "
                    kXR_PROTOCOLVSTRING, " version ", XrdVERSION);
-   pi->eDest->Say("++++++ xrootd protocol initialization started.");
+   pi->eDest->Say("++++++ xroot protocol initialization started.");
 
 // Return the protocol object to be used if static init succeeds
 //
    if (XrdXrootdProtocol::Configure(parms, pi))
       pp = (XrdProtocol *)new XrdXrootdProtocol();
       else txt = "failed.";
-    pi->eDest->Say("------ xrootd protocol initialization ", txt);
+    pi->eDest->Say("------ xroot protocol initialization ", txt);
    return pp;
 }
 }
 
 /******************************************************************************/
-/*                                                                            */
 /*           P r o t o c o l   P o r t   D e t e r m i n a t i o n            */
 /*                    X r d g e t P r o t o c o l P o r t                     */
 /******************************************************************************/
@@ -224,7 +224,7 @@ XrdSfsXioImpl SfsXioImpl(&XrdXrootdProtocol::Buffer,
 /******************************************************************************/
 
 XrdXrootdProtocol::XrdXrootdProtocol() 
-                    : XrdProtocol("xrootd protocol handler"),
+                    : XrdProtocol("xroot protocol handler"),
                       XrdSfsXio(SfsXioImpl),
                       ProtLink(this), Entity(0), AppName(0)
 {
@@ -466,6 +466,8 @@ int XrdXrootdProtocol::Process2()
                              if (!Request.header.dlen) return do_Truncate();
                              break;
           case kXR_query:    if (!Request.header.dlen) return do_Qfh();
+                             break;
+          case kXR_chkpoint: return do_ChkPnt();
           default:           break;
          }
 
@@ -748,7 +750,7 @@ int XrdXrootdProtocol::StatGen(struct stat &buf, char *xxBuff, int xxLen,
    if (buf.st_gid == myGID)
       {if (myGNLen >= xxLen) return m;
        strcpy(xxBuff, myGName);
-       n = myUNLen;
+       n = myGNLen;
       } else {
        if (!(n = XrdOucUtils::GidName(buf.st_gid,xxBuff,xxLen,keepT))) return m;
       }
@@ -875,16 +877,24 @@ int XrdXrootdProtocol::CheckSum(XrdOucStream *Stream, char **argv, int argc)
 {
    int rc, ecode;
 
-// The arguments must have <name> <cstype> <path> <tident> (i.e. argc >= 4)
+// The arguments must have <name> <cstype> <path> <tident> [name] (argc >= 4)
 //
    if (argc < 4)
       {Stream->PutLine("Internal error; not enough checksum args!");
        return 8;
       }
 
+// Construct the error information
+//
+   XrdOucEnv     myEnv;
+   XrdOucErrInfo myInfo(argv[3], &myEnv);
+
+// Add username, if present
+//
+   if (argc > 4 && *argv[4]) myEnv.Put("request.name", argv[4]);
+
 // Issue the checksum calculation (that's all we do here).
 //
-   XrdOucErrInfo myInfo(argv[3]);
    rc = osFS->chksum(XrdSfsFileSystem::csCalc, argv[1], argv[2], myInfo);
 
 // Return result regardless of what it is
